@@ -142,6 +142,12 @@ typedef struct rime_status_t {
   Bool is_ascii_punct;
 } RimeStatus;
 
+typedef struct rime_candidate_list_iterator_t {
+  void *ptr;
+  int index;
+  RimeCandidate candidate;
+} RimeCandidateListIterator;
+
 typedef struct rime_config_t {
   void* ptr;
 } RimeConfig;
@@ -252,6 +258,12 @@ RIME_API Bool RimeFreeContext(RimeContext* context);
 RIME_API Bool RimeGetStatus(RimeSessionId session_id, RimeStatus* status);
 RIME_API Bool RimeFreeStatus(RimeStatus* status);
 
+// Accessing candidate list
+
+RIME_API Bool RimeCandidateListBegin(RimeSessionId session_id, RimeCandidateListIterator* iterator);
+RIME_API Bool RimeCandidateListNext(RimeCandidateListIterator* iterator);
+RIME_API void RimeCandidateListEnd(RimeCandidateListIterator* iterator);
+
 // Runtime options
 
 RIME_API void RimeSetOption(RimeSessionId session_id, const char* option, Bool value);
@@ -484,7 +496,7 @@ typedef struct rime_api_t {
   //! caret posistion in terms of raw input
   size_t (*get_caret_pos)(RimeSessionId session_id);
 
-  //! select a candidate from current page
+  //! select a candidate at the given index in candidate list.
   Bool (*select_candidate)(RimeSessionId session_id, size_t index);
 
   //! get the version of librime
@@ -493,6 +505,14 @@ typedef struct rime_api_t {
   //! set caret posistion in terms of raw input
   void (*set_caret_pos)(RimeSessionId session_id, size_t caret_pos);
 
+  //! select a candidate from current page.
+  Bool (*select_candidate_on_current_page)(RimeSessionId session_id, size_t index);
+
+  // access candidate list.
+  Bool (*candidate_list_begin)(RimeSessionId session_id, RimeCandidateListIterator* iterator);
+  Bool (*candidate_list_next)(RimeCandidateListIterator* iterator);
+  void (*candidate_list_end)(RimeCandidateListIterator* iterator);
+
 } RimeApi;
 
 //! API entry
